' ScatterPlotVisuals.cs by Charles Petzold, December 2008

Imports Microsoft.VisualBasic
Imports System
Imports System.Collections
Imports System.Collections.Generic
Imports System.Collections.ObjectModel
Imports System.Collections.Specialized
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports System.Windows.Media

Namespace DataLibrary
	Public Class ScatterPlotVisuals
		Inherits FrameworkElement
		Private visualChildren As VisualCollection

        Public Shared ReadOnly ItemsSourceProperty As DependencyProperty = DependencyProperty.Register("ItemsSource", GetType(ObservableNotifiableCollection(Of DataPoint)), GetType(ScatterPlotVisuals), New PropertyMetadata(AddressOf OnItemsSourceChanged))

		Public Shared ReadOnly BrushesProperty As DependencyProperty = DependencyProperty.Register("Brushes", GetType(Brush()), GetType(ScatterPlotVisuals), New FrameworkPropertyMetadata(Nothing, FrameworkPropertyMetadataOptions.AffectsRender))

		Public Shared ReadOnly BackgroundProperty As DependencyProperty = Panel.BackgroundProperty.AddOwner(GetType(ScatterPlotRender))

		Public Sub New()
			visualChildren = New VisualCollection(Me)
			ToolTip = ""
		End Sub

		Public Property ItemsSource() As ObservableNotifiableCollection(Of DataPoint)
			Set(ByVal value As ObservableNotifiableCollection(Of DataPoint))
				SetValue(ItemsSourceProperty, value)
			End Set
			Get
				Return CType(GetValue(ItemsSourceProperty), ObservableNotifiableCollection(Of DataPoint))
			End Get
		End Property

		Public Property Brushes() As Brush()
			Set(ByVal value As Brush())
				SetValue(BrushesProperty, value)
			End Set
			Get
				Return CType(GetValue(BrushesProperty), Brush())
			End Get
		End Property

		Public Property Background() As Brush
			Set(ByVal value As Brush)
				SetValue(BackgroundProperty, value)
			End Set
			Get
				Return CType(GetValue(BackgroundProperty), Brush)
			End Get
		End Property

		Private Shared Sub OnItemsSourceChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
			TryCast(obj, ScatterPlotVisuals).OnItemsSourceChanged(args)
		End Sub

		Private Sub OnItemsSourceChanged(ByVal args As DependencyPropertyChangedEventArgs)
			visualChildren.Clear()

			If args.OldValue IsNot Nothing Then
				Dim coll As ObservableNotifiableCollection(Of DataPoint) = TryCast(args.OldValue, ObservableNotifiableCollection(Of DataPoint))
				RemoveHandler coll.CollectionCleared, AddressOf OnCollectionCleared
				RemoveHandler coll.CollectionChanged, AddressOf OnCollectionChanged
				RemoveHandler coll.ItemPropertyChanged, AddressOf OnItemPropertyChanged
			End If

			If args.NewValue IsNot Nothing Then
				Dim coll As ObservableNotifiableCollection(Of DataPoint) = TryCast(args.NewValue, ObservableNotifiableCollection(Of DataPoint))
				AddHandler coll.CollectionCleared, AddressOf OnCollectionCleared
				AddHandler coll.CollectionChanged, AddressOf OnCollectionChanged
				AddHandler coll.ItemPropertyChanged, AddressOf OnItemPropertyChanged

				CreateVisualChildren(coll)
			End If
		End Sub

		Private Sub OnCollectionCleared(ByVal sender As Object, ByVal args As EventArgs)
			RemoveVisualChildren(visualChildren)
		End Sub

		Private Sub OnCollectionChanged(ByVal sender As Object, ByVal args As NotifyCollectionChangedEventArgs)
			If args.OldItems IsNot Nothing Then
				RemoveVisualChildren(args.OldItems)
			End If

			If args.NewItems IsNot Nothing Then
				CreateVisualChildren(args.NewItems)
			End If
		End Sub

		Private Sub OnItemPropertyChanged(ByVal sender As Object, ByVal args As ItemPropertyChangedEventArgs)
			Dim dataPoint As DataPoint = TryCast(args.Item, DataPoint)

			For Each child As Visual In visualChildren
				Dim drawingVisual As DrawingVisualPlus = TryCast(child, DrawingVisualPlus)

				If dataPoint Is drawingVisual.DataPoint Then
					' Assume only VariableX or VariableY are changing
					Dim xform As TranslateTransform = TryCast(drawingVisual.Transform, TranslateTransform)

					If args.PropertyName = "VariableX" Then
						xform.X = RenderSize.Width * dataPoint.VariableX

					ElseIf args.PropertyName = "VariableY" Then
						xform.Y = RenderSize.Height * dataPoint.VariableY
					End If
				End If
			Next child
		End Sub

		Private Sub CreateVisualChildren(ByVal coll As ICollection)
			For Each obj As Object In coll
				Dim dataPoint As DataPoint = TryCast(obj, DataPoint)

				Dim drawingVisual As New DrawingVisualPlus()
				drawingVisual.DataPoint = dataPoint
				Dim dc As DrawingContext = drawingVisual.RenderOpen()

				dc.DrawEllipse(Brushes(dataPoint.Type), Nothing, New Point(0, 0), 1, 1)

				drawingVisual.Transform = New TranslateTransform(RenderSize.Width * dataPoint.VariableX, RenderSize.Height * dataPoint.VariableY)

				dc.Close()
				visualChildren.Add(drawingVisual)
			Next obj
		End Sub

		Private Sub RemoveVisualChildren(ByVal coll As ICollection)
			For Each obj As Object In coll
				Dim dataPoint As DataPoint = TryCast(obj, DataPoint)
				Dim removeList As List(Of DrawingVisualPlus) = New List(Of DrawingVisualPlus)()

				For Each child As Visual In visualChildren
					Dim drawingVisual As DrawingVisualPlus = TryCast(child, DrawingVisualPlus)
					If drawingVisual.DataPoint Is dataPoint Then
						removeList.Add(drawingVisual)
						Exit For
					End If
				Next child
				For Each drawingVisual As DrawingVisualPlus In removeList
					visualChildren.Remove(drawingVisual)
				Next drawingVisual
			Next obj
		End Sub

		Protected Overrides Sub OnRenderSizeChanged(ByVal sizeInfo As SizeChangedInfo)
			For Each child As Visual In visualChildren
				Dim drawingVisual As DrawingVisualPlus = TryCast(child, DrawingVisualPlus)
				Dim xform As TranslateTransform = TryCast(drawingVisual.Transform, TranslateTransform)

				If sizeInfo.WidthChanged Then
					xform.X = sizeInfo.NewSize.Width * drawingVisual.DataPoint.VariableX
				End If

				If sizeInfo.HeightChanged Then
					xform.Y = sizeInfo.NewSize.Height * drawingVisual.DataPoint.VariableY
				End If
			Next child
			MyBase.OnRenderSizeChanged(sizeInfo)
		End Sub

		Protected Overrides ReadOnly Property VisualChildrenCount() As Integer
			Get
				Return visualChildren.Count
			End Get
		End Property

		Protected Overrides Function GetVisualChild(ByVal index As Integer) As Visual
			If index < 0 OrElse index >= visualChildren.Count Then
				Throw New ArgumentOutOfRangeException("index")
			End If

			Return visualChildren(index)
		End Function

		Protected Overrides Sub OnRender(ByVal dc As DrawingContext)
			dc.DrawRectangle(Background, Nothing, New Rect(RenderSize))
		End Sub

		Protected Overrides Sub OnToolTipOpening(ByVal e As ToolTipEventArgs)
			Dim result As HitTestResult = VisualTreeHelper.HitTest(Me, Mouse.GetPosition(Me))

			If TypeOf result.VisualHit Is DrawingVisualPlus Then
				Dim drawingVisual As DrawingVisualPlus = TryCast(result.VisualHit, DrawingVisualPlus)
				Dim dataPoint As DataPoint = drawingVisual.DataPoint
				ToolTip = String.Format("{0}, X={1}, Y={2}", dataPoint.ID, dataPoint.VariableX, dataPoint.VariableY)
			End If
			MyBase.OnToolTipOpening(e)
		End Sub

		Protected Overrides Sub OnToolTipClosing(ByVal e As ToolTipEventArgs)
			ToolTip = ""
			MyBase.OnToolTipClosing(e)
		End Sub

		Private Class DrawingVisualPlus
			Inherits DrawingVisual
			Private privateDataPoint As DataPoint
			Public Property DataPoint() As DataPoint
				Get
					Return privateDataPoint
				End Get
				Set(ByVal value As DataPoint)
					privateDataPoint = value
				End Set
			End Property
		End Class
	End Class
End Namespace
